<?php
/** 
*
* @package install
* @version $Id: convert_vb30.php,v 1.34 2007/05/14 21:29:27 kellanved Exp $
* @copyright (c) 2006 phpBB Group 
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* NOTE to potential convertor authors. Please use this file to get
* familiar with the structure since we added some bare explanations here.
*
* Since this file gets included more than once on one page you are not able to add functions to it.
* Instead use a functions_ file.
*
* @ignore
*/
if (!defined('IN_PHPBB'))
{
	exit;
}

include($phpbb_root_path . 'config.' . $phpEx);
unset($dbpasswd);

/**
* $convertor_data provides some basic information about this convertor which is
* used on the initial list of convertors and to populate the default settings
*/
$convertor_data = array(
	'forum_name'	=> 'vB 3.0.x',
	'version'		=> '0.6',
	'phpbb_version'	=> '3.0.0',
	'author'		=> '<a href="http://wlx.westgis.ac.cn/">wlx</a>',
	'dbms'			=> $dbms,
	'dbhost'		=> $dbhost,
	'dbport'		=> $dbport,
	'dbuser'		=> $dbuser,
	'dbpasswd'		=> '',
	'dbname'		=> $dbname,
	'table_prefix'	=> 'vb_',
	'forum_path'	=> '../forums',
	'author_notes'	=> 'Please check forum permissions after conversion, and re-sync forums.',
);

/**
* $tables is a list of the tables (minus prefix) which we expect to find in the
* source forum. It is used to guess the prefix if the specified prefix is incorrect
*/
$tables = array(
	'access',
	'announcement',
	'attachment',
	'attachmenttype',
	'attachmentviews',
	'avatar',
	'bbcode',
	'forum',
	'forumpermission',
	'pm',
	'pmreceipt',
	'pmtext',
	'poll',
	'pollvote',
	'post', 
	'post_parsed', 
	'ranks',
	'smilie',
	'thread',
	'threadrate',
	'threadviews',
	//vb30 config schema
	'setting',
	'settinggroup',
	//vb30 user table
	'user',
	'useractivation',
	'userban',
	'userfield',
	'usergroup',
	'usergroupleader',
	'usergrouprequest',
	'usernote',
	'userpromotion',
	'usertextfield',
	'usertitle',
	// this table is empty in my vb303 forum.
	'word' 
);

/**
* $config_schema details how the board configuration information is stored in the source forum.
*
* 'table_format' can take the value 'file' to indicate a config file. In this case array_name
* is set to indicate the name of the array the config values are stored in
* 'table_format' can be an array if the values are stored in a table which is an assosciative array
* (as per phpBB 2.0.x)
* If left empty, values are assumed to be stored in a table where each config setting is
* a column (as per phpBB 1.x)
*
* In either of the latter cases 'table_name' indicates the name of the table in the database
*
* 'settings' is an array which maps the name of the config directive in the source forum
* to the config directive in phpBB3. It can either be a direct mapping or use a function.
* Please note that the contents of the old config value are passed to the function, therefore
* an in-built function requiring the variable passed by reference is not able to be used. Since
* empty() is such a function we created the function is_empty() to be used instead.
*/
$config_schema = array(
	'table_name'	=>	'setting',
	'table_format'	=>	array('varname' => 'value'),
	'settings'		=>	array(
		'allow_bbcode'			=> 'allowbbcode',
		'allow_smilies'			=> 'allowsmilies',
		'allow_sig'				=> 'allowsignatures',
		//no this value in vb30?
		//'allow_namechange'		=> 'allow_namechange',
		//'allow_avatar_local'	=> 'allow_avatar_local',
		//'allow_avatar_remote'	=> 'allow_avatar_remote',
		//'allow_avatar_upload'	=> 'allow_avatar_upload',
		'board_disable'			=> 'not(bbactive)',
		'sitename'				=> 'bbtitle',
		'site_desc'				=> 'description',
		//'session_length'		=> 'session_length',
		//'board_email_sig'		=> 'phpbb_set_encoding(board_email_sig)',
		'posts_per_page'		=> 'maxposts',
		'topics_per_page'		=> 'maxthreads',
		//unsure?
		'enable_confirm'		=> 'enableemail',
		//'board_email_form'		=> 'board_email_form',
		//'override_user_style'	=> 'override_user_style',
		//unsure, there are hotnumberposts and hotnumberviews, which one?
		'hot_threshold'			=> 'hotnumberposts',
		'max_poll_options'		=> 'maxpolloptions',
		'max_sig_chars'			=> 'sigmax',
		'limit_load'	=>	'loadlimit',
		//in vb30, pm_max_msgs is defined in usergroup.
		//'pm_max_msgs'			=> 'max_inbox_privmsgs',
		//smtp is not used in vb30.
		//'smtp_delivery'			=> 'smtp_delivery',
		//'smtp_host'				=> 'smtp_host',
		//'smtp_username'			=> 'smtp_username',
		//'smtp_password'			=> 'smtp_password',
		'require_activation'	=> 'verifyemail',
		'flood_interval'		=> 'floodchecktime',
		//'avatar_filesize'		=> 'avatar_filesize',
		//'avatar_max_width'		=> 'avatar_max_width',
		//'avatar_max_height'		=> 'avatar_max_height',
		//use phpbb default?		
		//'default_dateformat'	=> 'default_dateformat',
		'board_timezone'		=> 'timeoffset',
		//'allow_privmsg'			=> 'not(privmsg_disable)',
		'gzip_compress'			=> 'gziplevel',
		'coppa_enable'			=> 'usecoppa',
		'coppa_fax'				=> 'faxnumber',
		//'coppa_mail'			=> 'coppa_mail',
		//'record_online_users'	=> 'record_online_users',
		//'record_online_date'	=> 'record_online_date',
		//'board_startdate'		=> 'board_startdate',
	)
);

/**
* $test_file is the name of a file which is present on the source
* forum which can be used to check that the path specified by the 
* user was correct
*/
$test_file = 'showthread.php';

/**
* If this is set then we are not generating the first page of information but getting the conversion information.
*/
if (!$get_info)
{
	/**
	* Tests for further MODs can be included here.
	* Please use constants for this, prefixing them with MOD_
	*/

//	$src_db->sql_return_on_error(false);

	// Now let us set a temporary config variable for user id incrementing
	$sql = "SELECT userid
		FROM {$convert->src_table_prefix}user
		WHERE userid = 1";
	$result = $src_db->sql_query($sql);
	$user_id = (int) $src_db->sql_fetchfield('userid');
	$src_db->sql_freeresult($result);

	// If there is a user id 1, we need to increment user ids. :/
	if ($user_id === 1)
	{
		// Try to get the maximum user id possible...
		$sql = "SELECT MAX(userid) AS max_user_id
			FROM {$convert->src_table_prefix}user";
		$result = $src_db->sql_query($sql);
		$user_id = (int) $src_db->sql_fetchfield('max_user_id');
		$src_db->sql_freeresult($result);

		set_config('increment_user_id', ($user_id + 1), true);
	}
	else
	{
		set_config('increment_user_id', 0, true);
	}

	// Overwrite maximum
	//@define('DEFAULT_AVATAR_X_CUSTOM', get_config_value('avatar_max_width'));
	//@define('DEFAULT_AVATAR_Y_CUSTOM', get_config_value('avatar_max_height'));

/**
*	Description on how to use the convertor framework.
*
*	'schema' Syntax Description
*		-> 'target'			=> Target Table. If not specified the next table will be handled
*		-> 'primary'		=> Primary Key. If this is specified then this table is processed in batches
*		-> 'query_first'	=> array('target' or 'src', Query to execute before beginning the process
*								(if more than one then specified as array))
*		-> 'function_first'	=> Function to execute before beginning the process (if more than one then specified as array)
*								(This is mostly useful if variables need to be given to the converting process)
*		-> 'test_file'		=> This is not used at the moment but should be filled with a file from the old installation
*
*		// DB Functions
*		'distinct'	=> Add DISTINCT to the select query
*		'where'		=> Add WHERE to the select query
*		'group_by'	=> Add GROUP BY to the select query
*		'left_join'	=> Add LEFT JOIN to the select query (if more than one joins specified as array)
*		'having'	=> Add HAVING to the select query
*
*		// DB INSERT array
*		This one consist of three parameters
*		First Parameter: 
*							The key need to be filled within the target table
*							If this is empty, the target table gets not assigned the source value
*		Second Parameter:
*							Source value. If the first parameter is specified, it will be assigned this value.
*							If the first parameter is empty, this only gets added to the select query
*		Third Parameter:
*							Custom Function. Function to execute while storing source value into target table. 
*							The functions return value get stored.
*							The function parameter consist of the value of the second parameter.
*
*							types:
*								- empty string == execute nothing
*								- string == function to execute
*								- array == complex execution instructions
*		
*		Complex execution instructions:
*		@todo test complex execution instructions - in theory they will work fine
*
*							By defining an array as the third parameter you are able to define some statements to be executed. The key
*							is defining what to execute, numbers can be appended...
*
*							'function' => execute function
*							'execute' => run code, whereby all occurrences of {VALUE} get replaced by the last returned value.
*										The result *must* be assigned/stored to {RESULT}.
*							'typecast'	=> typecast value
*
*							The returned variables will be made always available to the next function to continue to work with.
*
*							example (variable inputted is an integer of 1):
*
*							array(
*								'function1'		=> 'increment_by_one',		// returned variable is 2
*								'typecast'		=> 'string',				// typecast variable to be a string
*								'execute'		=> '{RESULT} = {VALUE} . ' is good';', // returned variable is '2 is good'
*								'function2'		=> 'replace_good_with_bad',				// returned variable is '2 is bad'
*							),
*
*/

	$convertor = array(
		'test_file'				=> 'showthread.php',

		'avatar_path'			=> 'images/avatars/',
		'avatar_gallery_path'	=> 'images/avatars/',
		'smilies_path'			=> 'images/smilies/',
		'upload_path'			=> 'images/attach/',
		'thumbnails'			=> '',
		'ranks_path'			=> false, // phpBB 2.0.x had no config value for a ranks path

		// We empty some tables to have clean data available
		'query_first'			=> array(
			array('target', $convert->truncate_statement . SEARCH_RESULTS_TABLE),
			array('target', $convert->truncate_statement . SEARCH_WORDLIST_TABLE),
			array('target', $convert->truncate_statement . SEARCH_WORDMATCH_TABLE),
			array('target', $convert->truncate_statement . LOG_TABLE),
		),


		// phpBB2 allowed some similar usernames to coexist which would have the same
		// username_clean in phpBB3 which is not possible, so we'll give the admin a list
		// of user ids and usernames and let him deicde what he wants to do with them
		'execute_first'	=> '
			phpbb_check_username_collisions(); //检查重名的用户？vb30是否需要？
			import_avatar_gallery();
			phpbb_insert_forums();
		',

		'execute_last'	=> array('
			add_bots();
		', '
			vb_import_polloption();
		', '
			vb_import_customavatar();
		', '
			update_folder_pm_count();
		', '
			update_unread_count();
		', '
			phpbb_convert_authentication(\'start\');
		', '
			phpbb_convert_authentication(\'first\');
		', '
			phpbb_convert_authentication(\'second\');
		', '
			phpbb_convert_authentication(\'third\');
		'),

		'schema' => array(

			array(
				'target'		=> ATTACHMENTS_TABLE,
				'primary'		=> 'attachment.attachmentid',
				'query_first'	=>  array('target', $convert->truncate_statement . ATTACHMENTS_TABLE),
				'autoincrement'	=> 'attach_id',

				array('attach_id',				'attachment.attachmentid',				''),
				array('post_msg_id',			'attachment.postid',					''),
				array('topic_id',				'post.threadid',						''),
				array('in_message',				0,										''),
				array('is_orphan',				0,										''),
				array('poster_id',				'attachment.userid AS poster_id',	'phpbb_user_id'),
				array('physical_filename',		'attachment.userid',	'vb_import_attachment'),
				array('real_filename',			'attachment.filename',	''),
				array('download_count',			'attachment.counter',		''),
				array('extension',				'attachment.filename',			'vb_file_ext'),
				array('mimetype',				'attachment.filename',			'vb_file_mime'),
				array('filesize',				'attachment.filesize',			''),
				array('filetime',				'attachment.dateline',			''),//vb3中是上传时间还是文件时间？

				'where'			=> 'post.postid = attachment.postid',
			),


			array(
				'target'		=> BANLIST_TABLE,
				'query_first'	=> array('target', $convert->truncate_statement . BANLIST_TABLE),

				array('ban_userid',				'userban.userid',				'phpbb_user_id'),
				array('ban_reason',				'',									''),
				array('ban_give_reason',		'',									''),

			),


			array(
				'target'		=> TOPICS_TABLE,
				'query_first'	=> array(
							array('target', $convert->truncate_statement . TOPICS_TABLE),
							),
				'primary'		=> 'thread.threadid',
				'autoincrement'	=> 'topic_id',

				array('topic_id',				'thread.threadid',				''),
				array('forum_id',				'thread.forumid',					''),
				array('icon_id',				'thread.iconid',					''),
				array('topic_poster',		'thread.postuserid',	'phpbb_user_id'),
				array('topic_attachment',	'thread.attach', 					''),
				array('topic_title',			'thread.title',					'vb_title'),
				array('topic_time',			'thread.dateline',				''),
				array('topic_views',			'thread.views',					''),
				array('topic_replies',		'thread.replycount',				''),
				array('topic_replies_real',		'thread.replycount',				''),
				array('topic_last_post_id',		'',		''),//?
				array('topic_status',			'not(thread.open)',				''),
				array('topic_moved_id',			0,									''),
				array('topic_type',				'thread.sticky',				''),
				array('topic_first_post_id',	'thread.firstpostid',		''),
				array('topic_first_poster_name',	'thread.postusername',		''),
				array('topic_last_poster_name',	'thread.lastposter',		''),
				array('topic_last_post_time',	'thread.lastpost',			''),

				array('poll_title',				'poll.question',				array('function1' => 'null_to_str', 'function2' => 'phpbb_set_encoding', 'function3' => 'utf8_htmlspecialchars')),
				array('poll_start',				'poll.dateline',				''),
				array('poll_length',			'poll.timeout',			'vb_poll_length'),
				array('',	'poll.numberoptions',	''),
				array('poll_max_options',		'poll.multiple',			'vb_poll_options'),//二者实现的方法不同
				array('poll_vote_change',		0,									''),

				'left_join'		=> 'thread LEFT JOIN poll ON thread.pollid = poll.pollid',
				//'where'			=> 'topics.topic_moved_id = 0',
			),


			array(
				'target'		=> TOPICS_WATCH_TABLE,
				'primary'		=> 'subscribethread.subscribethreadid',
				'query_first'	=> array('target', $convert->truncate_statement . TOPICS_WATCH_TABLE),

				array('topic_id',				'subscribethread.threadid',			''),
				array('user_id',				'subscribethread.userid',				'phpbb_user_id'),
				array('notify_status',			'subscribethread.emailupdate',		''),//?
			),
//unsure how to deal this smiles.
/*
			array(
				'target'		=> SMILIES_TABLE,
				'query_first'	=> array('target', $convert->truncate_statement . SMILIES_TABLE),
				'autoincrement'	=> 'smiley_id',

				array('smiley_id',				'smilie.smilie_id',				''),
				array('code',					'smilie.smiletext',						'phpbb_set_encoding'),
				array('emotion',				'smilie.title',					'phpbb_set_encoding'),
				array('smiley_url',				'smilie.smilepath',				'import_smiley'),
				array('smiley_width',			'smilie.smilepath',				'get_smiley_width'),
				array('smiley_height',			'smilie.smilepath',				'get_smiley_height'),
				array('smiley_order',			'smilie.smilieid',				''),
				array('display_on_posting',		'smilie.smilie_id',				array(
					'execute'		=> '{RESULT} = ({VALUE}[0] <= 20) ? 1 : 0;',
				)),

				'order_by'		=> 'smilie.smilie_id ASC',
			),
*/
//poll选项独立作为一个函数来进行处理？					array('target','ALTER TABLE '. USERS_TABLE . ' DROP COLUMN goodnees'),
					//array('target','ALTER TABLE '. USERS_TABLE . ' ADD COLUMN salt varchar(3)'),


			array(
				'target'		=> POLL_VOTES_TABLE,
				'primary'		=> 'pollvote.pollvoteid',
				'query_first'	=> array('target', $convert->truncate_statement . POLL_VOTES_TABLE),

				array('poll_option_id',			'pollvote.voteoption',									'vb_voteoption'),
				array('topic_id',				'thread.threadid',				''),
				array('vote_user_id',			'pollvote.userid',			'phpbb_user_id'),
				array('vote_user_ip',			'',			''),

				'left_join'		=> 'pollvote LEFT JOIN thread ON pollvote.pollid = thread.pollid',
			),

			array(
				'target'		=> POSTS_TABLE,
				'primary'		=> 'post.postid',
				'autoincrement'	=> 'postid',
				'query_first'	=> array('target', $convert->truncate_statement . POSTS_TABLE),
				'execute_first'	=> '
					$config["max_post_chars"] = -1;
					$config["max_quote_depth"] = 0;
				',

				array('post_id',				'post.postid',					''),
				array('topic_id',				'post.threadid',					''),
				array('forum_id',				'thread.forumid',					''),
				array('poster_id',			'post.userid as poster_id',					'phpbb_user_id'),
				array('icon_id',				'post.iconid',									''),
				array('poster_ip',				'post.ipaddress',					''),
				array('post_time',				'post.dateline',					''),
				array('enable_bbcode',			1,				''),
				//array('',						'posts.enable_html',				''),
				array('enable_smilies',			'post.allowsmilie  AS enable_smilies',				''),
				array('enable_sig',				'post.showsignature',					''),
				array('enable_magic_url',		1,									''),
				array('post_username',			'post.username',				'phpbb_set_encoding'),
				array('post_subject',			'post.title',			'phpbb_set_encoding'),
				array('post_attachment',		'post.attach', ''),
				//array('post_edit_time',			'posts.post_edit_time',				array('typecast' => 'int')),
				//array('post_edit_count',		'posts.post_edit_count',			''),
				//array('post_edit_reason',		'',									''),
				//array('post_edit_user',			'',									'phpbb_post_edit_user'),

				array('bbcode_uid',				'post.dateline  AS post_time',					'make_uid'),
				array('post_text',				'post.pagetext',				'phpbb_prepare_message'),
				//array('',						'posts_text.bbcode_uid AS old_bbcode_uid',			''),
				array('bbcode_bitfield',		'',									'get_bbcode_bitfield'),
				array('post_checksum',			'',									''),

				// Commented out inline search indexing, this takes up a LOT of time. :D
				// @todo We either need to enable this or call the rebuild search functionality post convert
/*				array('',						'',									'search_indexing'),
				array('',						'posts_text.post_text AS message',	''),
				array('',						'posts_text.post_subject AS title',	''),*/

				'left_join'		=> 'post LEFT JOIN thread ON post.threadid = thread.threadid',
				//'where'			=>	'post.threadid = posts_text.post_id'
			),

			array(
				'target'		=> PRIVMSGS_TABLE,
				'primary'		=> 'pmtextid',
				'autoincrement'	=> 'msg_id',
				'query_first'	=> array(
					array('target', $convert->truncate_statement . PRIVMSGS_TABLE),
					array('target', $convert->truncate_statement . PRIVMSGS_RULES_TABLE),
				),

				'execute_first'	=> '
					$config["max_post_chars"] = -1;
					$config["max_quote_depth"] = 0;
				',

				array('msg_id',					'pmtext.pmtextid',				''),
				array('root_level',				0,									''),
				array('author_id',				'pmtext.fromuserid AS poster_id',	'phpbb_user_id'),
				array('icon_id',				'pmtext.iconid',									''),
				//array('author_ip',				'privmsgs.privmsgs_ip',				'decode_ip'),
				array('message_time',			'pmtext.dateline',			''),
				//array('enable_bbcode',			'privmsgs.privmsgs_enable_bbcode AS enable_bbcode',	''),
				//array('',						'privmsgs.privmsgs_enable_html AS enable_html',	''),
				array('enable_smilies',			'pmtext.allowsmilie AS enable_smilies',	''),
				array('enable_magic_url',		1,									''),
				array('enable_sig',				'pmtext.showsignature',		''),
				array('message_subject',		'pmtext.title',		'phpbb_set_encoding'), 
				array('message_edit_reason',	'',									''),
				array('message_edit_user',		0,									''),
				array('message_edit_time',		0,									''),
				array('message_edit_count',		0,									''),

				array('bbcode_uid',				'pmtext.dateline AS post_time',	'make_uid'),
				array('message_text',			'pmtext.message',			'phpbb_prepare_message'),
				//array('',						'privmsgs_text.privmsgs_bbcode_uid AS old_bbcode_uid',			''),
				array('bbcode_bitfield',		'',										'get_bbcode_bitfield'),
				array('to_address',				'pmtext.touserarray',			'vb_privmsgs_touserarray'),
				array('bcc_address',			'',										''),

			),

			array(
				'target'		=> PRIVMSGS_FOLDER_TABLE,
				'primary'		=> 'user.userid',
				'query_first'	=> array('target', $convert->truncate_statement . PRIVMSGS_FOLDER_TABLE),

				array('user_id',				'user.userid',						'phpbb_user_id'),
				array('folder_name',			$user->lang['CONV_SAVED_MESSAGES'],		''),
				array('pm_count',				0,										''),
			
			),

			// Inbox
			array(
				'target'		=> PRIVMSGS_TO_TABLE,
				'primary'		=> 'pm.pmid',
				'query_first'	=> array('target', $convert->truncate_statement . PRIVMSGS_TO_TABLE),

				array('msg_id',					'pm.pmtextid',					''),
				array('user_id',					'pm.userid',			'phpbb_user_id'),
				array('author_id',				'pmtext.fromuserid',		'phpbb_user_id'),
				array('pm_deleted',				0,										''),
				array('pm_new',					'pm.messageread',				'vb_unread_pm'),
				array('pm_unread',				'pm.messageread',				'vb_unread_pm'),
				array('pm_replied',				'pm.messageread',				'vb_replied_pm'),
				array('pm_marked',				0,										''),
				array('pm_forwarded',			0,										''),
				array('folder_id',				PRIVMSGS_INBOX,							''),

				'left_join'		=> 'pm LEFT JOIN pmtext ON pm.pmtextid = pmtext.pmtextid',
				'where'			=> 'pm.folderid = 0',
			),
		
			// Sentbox
			array(
				'target'		=> PRIVMSGS_TO_TABLE,
				'primary'		=> 'pm.pmid',

				array('msg_id',					'pm.pmtextid',					''),
				array('user_id',					'pm.userid',		'phpbb_user_id'),
				array('author_id',				'pmtext.fromuserid',		'phpbb_user_id'),
				array('pm_deleted',				0,										''),
				array('pm_new',					0,				''),
				array('pm_unread',				0,				''),
				array('pm_replied',				'pm.messageread',					'vb_replied_pm'),
				array('pm_marked',				0,										''),
				array('pm_forwarded',			0,										''),
				array('folder_id',				PRIVMSGS_SENTBOX,					''),

				'left_join'		=> 'pm LEFT JOIN pmtext ON pm.pmtextid = pmtext.pmtextid',
				'where'			=> 'pm.folderid = -1',
			),

			array(
				'target'		=> RANKS_TABLE,
				'autoincrement'	=> 'rank_id',
				'query_first'	=> array('target', $convert->truncate_statement . RANKS_TABLE),

				array('rank_id',				'usertitle.usertitleid',					''),
				array('rank_title',				'usertitle.title',				''),
				array('rank_min',			'minposts',									''),
				array('rank_special',			0,									''),

			),

			array(
				'target'		=> GROUPS_TABLE,
				'autoincrement'	=> 'group_id',
				'query_first'	=> array('target', $convert->truncate_statement . GROUPS_TABLE),

				array('group_id',				'usergroup.usergroupid',					''),
				array('group_type',				'usergroup.ispublicgroup',				'phpbb_convert_group_type'),
				array('group_display',			0,									''),
				array('group_legend',			0,									''),
				array('group_name',				'usergroup.title',				'phpbb_convert_group_name'), 
				array('group_desc',				'usergroup.description',			'phpbb_set_encoding'),

				'where'			=> 'usergroup.usergroupid > 1',
			),


			array(
				'target'		=> USER_GROUP_TABLE,
				'query_first'	=> array('target', $convert->truncate_statement . USER_GROUP_TABLE),
				'execute_first'	=> '
					add_default_groups();
				',

				array('group_id',		'user.usergroupid',					''),
				array('user_id',		'user.userid',			'phpbb_user_id'),
				array('group_leader',	0,									''),
				array('user_pending',	0,									''),

				//'where'			=> 'groups.group_single_user = 0 AND groups.group_moderator <> 0',
			),
//此处偷工减料
//正常情况下应该分解membergroupids
//我的站点上只有两个用户有额外组，且是单组
			array(
				'target'		=> USER_GROUP_TABLE,

				array('group_id',		'user.membergroupids',				''),
				array('user_id',		'user.userid',				'phpbb_user_id'),
				array('group_leader',	0,									''),
				array('user_pending',	0,			''),

				'where'			=> 'user.membergroupids <>""',
			),

			array(
				'target'		=> USERS_TABLE,
				'primary'		=> 'user.userid',
				'autoincrement'	=> 'user_id',
				'query_first'	=> array(
					array('target', 'DELETE FROM ' . USERS_TABLE . ' WHERE user_id <> ' . ANONYMOUS),
					array('target', $convert->truncate_statement . BOTS_TABLE),
					//array('target','ALTER TABLE '. USERS_TABLE . ' DROP COLUMN salt'),
					array('target','ALTER TABLE '. USERS_TABLE . ' ADD COLUMN salt varchar(3)'),
				),

				'execute_last'	=> '
					remove_invalid_users();
				',

				array('user_id',				'user.userid',					'phpbb_user_id'),
				array('',						'user.userid AS poster_id',		'phpbb_user_id'),
				//array('user_type',				'users.user_active',				'set_user_type'),//?
				array('group_id',				'user.usergroupid',					'phpbb_set_primary_group'),
				array('user_ip',				'user.ipaddress',					''),
				array('salt',				'user.salt',					''),
				array('user_regdate',			'user.joindate',				''),
				array('username',				'user.username',					''), 
				array('username_clean',			'user.username',					array('function1' => 'phpbb_set_default_encoding', 'function2' => 'utf8_clean_string')),
				array('user_password',			'user.password',				''),
				array('user_pass_convert',		0,									''),
				array('user_posts',				'user.posts',					''),
				array('user_email',				'user.email',					'strtolower'),
				array('user_email_hash',		'user.email',					'gen_email_hash'),
				array('user_birthday',			'user.birthday',	''),//?phpbb_get_birthday
				array('user_lastvisit',			'user.lastvisit',				''),
				array('user_lastmark',			'user.lastactivity',				''),
				array('user_lang',				$config['default_lang'],			''),
				//array('',						'users.user_lang',					''),
				array('user_timezone',			'user.timezoneoffset',				''),
				//array('user_dateformat',		'users.user_dateformat',			''),//?
				array('user_inactive_reason',	'',									'phpbb_inactive_reason'),
				array('user_inactive_time',		'',									'phpbb_inactive_time'),

				array('user_interests',			'userfield.field3',				array('function1' => 'phpbb_set_encoding')),
				array('user_occ',				'userfield.field4',					array('function1' => 'phpbb_set_encoding')),
				array('user_website',			'user.homepage',				'validate_website'),
				array('user_jabber',			'',									''),
				array('user_msnm',				'user.msn',					''),
				array('user_yim',				'user.yahoo',					''),
				array('user_aim',				'user.aim',					''),
				array('user_icq',				'user.icq',					''),
				array('user_from',				'userfield.field2',					array('function1' => 'phpbb_set_encoding')),
				array('user_rank',				'',					''),
				array('user_permissions',		'',									''),

				array('user_avatar',			'avatar.avatarpath',				'vb_avatar'),
				array('user_avatar_type',		AVATAR_GALLERY,			''),
				array('user_avatar_width',		'avatar.avatarpath',				'vb_get_avatar_width'),
				array('user_avatar_height',		'avatar.avatarpath',				'vb_get_avatar_height'),

				//array('user_new_privmsg',		'users.user_new_privmsg',			''),
				array('user_unread_privmsg',	'user.pmunread',									''), 
				//array('user_last_privmsg',		'users.user_last_privmsg',			''),
				array('user_emailtime',			'user.emailstamp',				'null_to_zero'),
				array('user_notify',			'user.autosubscribe',				''),
				array('user_notify_pm',			'user.autosubscribe',				''),
				array('user_notify_type',		NOTIFY_EMAIL,						''),
				//array('user_allow_pm',			'users.user_allow_pm',				''),
				//array('user_allow_viewonline',	'users.user_allow_viewonline',		''),
				//array('user_allow_viewemail',	'users.user_viewemail',				''),
				//array('user_actkey',			'users.user_actkey',				''),
				array('user_newpasswd',			'',									''), // Users need to re-request their password...
				array('user_style',				$config['default_style'],			''),

				array('user_options',			'',									'set_user_options'),
				//array('',						'users.user_popup_pm AS popuppm',	''),
				//array('',						'users.user_allowhtml AS html',		''),
				//array('',						'users.user_allowbbcode AS bbcode',	''),
				//array('',						'users.user_allowsmile AS smile',	''),
				//array('',						'users.user_attachsig AS attachsig',''),

				array('user_sig_bbcode_uid',		'user.joindate',							'make_uid'),
				array('user_sig',					'usertextfield.signature',								'phpbb_prepare_message'),
				//array('',							'users.user_sig_bbcode_uid AS old_bbcode_uid',	''),
				array('user_sig_bbcode_bitfield',	'',												'get_bbcode_bitfield'),
				array('',							'user.joindate AS post_time',				''),

				'left_join'		=> array('user LEFT JOIN userfield ON user.userid = userfield.userid','user left join usertextfield on user.userid=usertextfield.userid','user left join avatar on user.avatarid=avatar.avatarid'),
				'where'			=> 'user.userid <> -1',
			),
		),
	);
}

?>
